from mock import *  # NOQA
from nose.tools import *  # NOQA

from method_override.middleware import MethodOverrideMiddleware


class TestMethodOverrideMiddleware(object):
    def test_noop(self):
        request = MagicMock()
        request.method = 'POST'
        request.POST = {}
        middleware = MethodOverrideMiddleware()
        middleware.process_view(request, None, [], {})
        assert_equal(request.method, 'POST')

    def test_get_param_noop(self):
        request = MagicMock()
        request.method = 'GET'
        request.POST = {'_method': 'PUT'}
        middleware = MethodOverrideMiddleware()
        middleware.process_view(request, None, [], {})
        assert_equal(request.method, 'GET')

    def test_unsupported_method(self):
        request = MagicMock()
        request.method = 'POST'
        request.POST = {'_method': 'DESTROY'}
        middleware = MethodOverrideMiddleware()
        middleware.process_view(request, None, [], {})
        assert_equal(request.method, 'POST')

    def test_param_put_override(self):
        request = MagicMock()
        request.method = 'POST'
        request.POST = {'_method': 'PUT'}
        middleware = MethodOverrideMiddleware()
        middleware.process_view(request, None, [], {})
        assert_equal(request.method, 'PUT')

    def test_param_put_query_dict(self):
        request = MagicMock()
        request.method = 'POST'
        request.POST = {'_method': 'PUT', 'name': 'Bob'}
        middleware = MethodOverrideMiddleware()
        middleware.process_view(request, None, [], {})
        assert_equal(request.PUT.get('name'), 'Bob')

    def test_lowercase_param(self):
        request = MagicMock()
        request.method = 'POST'
        request.POST = {'_method': 'put'}
        middleware = MethodOverrideMiddleware()
        middleware.process_view(request, None, [], {})
        assert_equal(request.method, 'PUT')

    def test_header_override(self):
        request = MagicMock()
        request.method = 'POST'
        request.POST = {}
        request.META = {'HTTP_X_HTTP_METHOD_OVERRIDE': 'PUT'}
        middleware = MethodOverrideMiddleware()
        middleware.process_view(request, None, [], {})
        assert_equal(request.method, 'PUT')



from mock import *  # NOQA
from nose.tools import *  # NOQA

from method_override.templatetags.method_override import method_override


class TestTemplateTags(object):
    def test_method_override(self):
        result = method_override('PUT')
        assert_equal(
            result, '<input type="hidden" name="_method" value="PUT">'
        )

    def test_method_override_marked_safe(self):
        result = method_override('PUT')
        assert_true(hasattr(result, '__html__'))



from method_override import settings


__all__ = ['MethodOverrideMiddleware']


class MethodOverrideMiddleware(object):
    def process_view(self, request, callback, callback_args, callback_kwargs):
        if request.method != 'POST':
            return
        method = self._get_method_override(request)
        if method in settings.ALLOWED_HTTP_METHODS:
            setattr(request, method, request.POST.copy())
            request.method = method

    def _get_method_override(self, request):
        method = (request.POST.get(settings.PARAM_KEY) or
                  request.META.get(settings.HTTP_HEADER))
        return method and method.upper()
